@@ -1,8 +1,5 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import logging
 import numpy as np
-import six
 
 from madminer.utils.interfaces.madminer_hdf5 import load_madminer_settings, madminer_event_loader
 from madminer.utils.morphing import PhysicsMorpher, NuisanceMorpher
@@ -11,7 +8,7 @@ from madminer.utils.various import format_benchmark, mdot
 logger = logging.getLogger(__name__)
 
 
-class DataAnalyzer(object):
+class DataAnalyzer:
     """
     Collects common functionality that is used when analysing data in the MadMiner file.
 
@@ -115,7 +112,7 @@ class DataAnalyzer(object):
             at the closest benchmark point to a given parameter point.
 
         return_sampling_ids : bool, optional
-            If True, the iterator returns the sampling IDs in additioin to observables and weights.
+            If True, the iterator returns the sampling IDs in addition to observables and weights.
 
         Yields
         ------
@@ -134,7 +131,7 @@ class DataAnalyzer(object):
             include_nuisance_parameters = self.include_nuisance_parameters
 
         sampling_benchmark = self._find_closest_benchmark(generated_close_to)
-        logger.debug("Sampling benchmark closest to %s: %s", generated_close_to, sampling_benchmark)
+        logger.debug(f"Sampling benchmark closest to {generated_close_to}: {sampling_benchmark}")
 
         if sampling_benchmark is None:
             sampling_factors = self._calculate_sampling_factors()
@@ -208,22 +205,28 @@ class DataAnalyzer(object):
         """
 
         x, weights_benchmarks = next(
-            self.event_loader(batch_size=None, start=start_event, end=end_event, generated_close_to=generated_close_to)
+            self.event_loader(
+                start=start_event,
+                end=end_event,
+                batch_size=None,
+                generated_close_to=generated_close_to,
+            )
         )
 
         # Pick events randomly
         n_events = len(x)
+
         if n_draws is not None and n_draws < n_events:
             idx = np.random.choice(n_events, n_draws, replace=False)
             x = x[idx]
             weights_benchmarks = weights_benchmarks[idx]
         elif n_draws is not None:
-            logger.warning("Requested %s events, but only %s available", n_draws, n_events)
+            logger.warning(f"Requested {n_draws} events, but only {n_events} available")
 
         # Process and return appropriate weights
         if theta is None:
             return x, weights_benchmarks
-        elif isinstance(theta, six.string_types):
+        elif isinstance(theta, str):
             i_benchmark = list(self.benchmarks.keys()).index(theta)
             return x, weights_benchmarks[:, i_benchmark]
         elif derivative:
@@ -234,7 +237,7 @@ class DataAnalyzer(object):
         else:
             # TODO: nuisance params
             if nu is not None:
-                raise NotImplementedError
+                raise NotImplementedError()
             theta_matrix = self._get_theta_benchmark_matrix(theta)
             weights_theta = mdot(theta_matrix, weights_benchmarks)
             return x, weights_theta
@@ -312,7 +315,7 @@ class DataAnalyzer(object):
                 partition, test_split, validation_split
             )
         else:
-            raise ValueError("Events has to be either 'all', 'train', or 'test', but got {}!".format(partition))
+            raise ValueError(f"Invalid partition type: {partition}")
 
         # Theta matrices (translation of benchmarks to theta, at nominal nuisance params)
         if thetas is None:
@@ -360,7 +363,8 @@ class DataAnalyzer(object):
 
         if n_events == 0:
             raise RuntimeError(
-                "Did not find events with test_split = %s and generated_close_to = %s", test_split, generated_close_to
+                f"Did not find events with test_split = {test_split} "
+                f"and generated_close_to = {generated_close_to}"
             )
 
         xsec_uncertainties = np.maximum(xsec_uncertainties, 0.0) ** 0.5
@@ -401,11 +405,14 @@ class DataAnalyzer(object):
              account. Otherwise, the list has to have the same number of elements as thetas, and each entry can specify
              nuisance parameters at nominal value (None) or a value of the nuisance parameters (ndarray).
 
+        partition : {"train", "test", "validation", "all"}, optional
+            Which events to use. Default: "all".
+
         test_split : float, optional
             Fraction of events reserved for testing. Default value: 0.2.
 
-        partition : {"train", "test", "validation", "all"}, optional
-            Which events to use. Default: "all".
+        validation_split : float, optional
+            Fraction of weighted events reserved for validation. Default value: 0.2.
 
         gradients : {"all", "theta", "nu"}, optional
             Which gradients to calculate. Default value: "all".
@@ -423,7 +430,7 @@ class DataAnalyzer(object):
             Calculated cross section gradients in pb with shape (n_gradients,).
         """
 
-        logger.debug("Calculating cross section gradients for thetas = %s and nus = %s", thetas, nus)
+        logger.debug(f"Calculating cross section gradients for thetas = {thetas} and nus = {nus}")
 
         # Inputs
         include_nuisance_benchmarks = nus is not None or gradients in ["all", "nu"]
@@ -431,7 +438,7 @@ class DataAnalyzer(object):
             nus = [None for _ in thetas]
         assert len(nus) == len(thetas), "Numbers of thetas and nus don't match!"
         if gradients not in ["all", "theta", "nu"]:
-            raise RuntimeError("Gradients has to be 'all', 'theta', or 'nu', but got {}".format(gradients))
+            raise RuntimeError(f"Invalid gradients type: {gradients}")
 
         # Which events to use
         if partition == "all":
@@ -442,12 +449,13 @@ class DataAnalyzer(object):
                 partition, test_split, validation_split
             )
         else:
-            raise ValueError("Events has to be either 'all', 'train', or 'test', but got {}!".format(partition))
+            raise ValueError(f"Invalid partition type: {partition}")
 
         # Theta matrices (translation of benchmarks to theta, at nominal nuisance params)
         theta_matrices = np.asarray(
             [self._get_theta_benchmark_matrix(theta) for theta in thetas]
         )  # shape (n_thetas, n_benchmarks)
+
         theta_gradient_matrices = np.asarray(
             [self._get_dtheta_benchmark_matrix(theta) for theta in thetas]
         )  # shape (n_thetas, n_gradients, n_benchmarks)
@@ -465,7 +473,7 @@ class DataAnalyzer(object):
             )
         ):
             n_batch, _ = benchmark_weights.shape
-            logger.debug("Batch %s with %s events", i_batch + 1, n_batch)
+            logger.debug(f"Batch {i_batch+1} with {n_batch} events")
 
             if gradients in ["all", "theta"]:
                 nom_gradients = mdot(
@@ -512,33 +520,35 @@ class DataAnalyzer(object):
             )
 
     def _report_setup(self):
-        logger.info("Found %s parameters", self.n_parameters)
-        for i, (key, values) in enumerate(six.iteritems(self.parameters)):
-            logger.info("  %s: %s (%s)", i, key, " / ".join(str(x) for x in values))
+        logger.info(f"Found {self.n_parameters} parameters")
+        for i, (key, values) in enumerate(self.parameters.items()):
+            values_str = " / ".join(str(x) for x in values)
+            logger.info(f"  {i}: {key} ({values_str})")
 
         if self.nuisance_parameters is not None:
-            logger.info("Found %s nuisance parameters", self.n_nuisance_parameters)
-            for i, (key, values) in enumerate(six.iteritems(self.systematics)):
-                logger.info("  %s: %s (%s)", i, key, " / ".join(str(x) for x in values))
+            logger.info(f"Found {self.n_nuisance_parameters} nuisance parameters")
+            for i, (key, values) in enumerate(self.systematics.items()):
+                values_str = " / ".join(str(x) for x in values)
+                logger.info(f"  {i}: {key} ({values_str})")
         else:
             logger.info("Did not find nuisance parameters")
             self.include_nuisance_parameters = False
 
-        logger.info("Found %s benchmarks, of which %s physical", self.n_benchmarks, self.n_benchmarks_phys)
-        for (key, values), is_nuisance in zip(six.iteritems(self.benchmarks), self.benchmark_is_nuisance):
+        logger.info(f"Found {self.n_benchmarks} benchmarks")
+        for (key, values), is_nuisance in zip(self.benchmarks.items(), self.benchmark_is_nuisance):
             if is_nuisance:
                 logger.debug("   %s: systematics", key)
             else:
                 logger.debug("   %s: %s", key, format_benchmark(values))
 
-        logger.info("Found %s observables", self.n_observables)
+        logger.info(f"Found {self.n_observables} observables")
         if self.observables is not None:
             for i, obs in enumerate(self.observables):
                 logger.debug("  %2.2s %s", i, obs)
 
-        logger.info("Found %s events", self.n_samples)
+        logger.info(f"Found {self.n_samples} events")
         if self.n_events_generated_per_benchmark is not None:
-            for events, name in zip(self.n_events_generated_per_benchmark, six.iterkeys(self.benchmarks)):
+            for events, name in zip(self.n_events_generated_per_benchmark, self.benchmarks.keys()):
                 if events > 0:
                     logger.info("  %s signal events sampled from benchmark %s", events, name)
             if self.n_events_backgrounds is not None and self.n_events_backgrounds > 0:
@@ -572,17 +582,19 @@ class DataAnalyzer(object):
         matrix = np.zeros(
             (self.n_benchmarks, self.n_parameters, self.n_benchmarks)
         )  # (n_thetas, n_gradients, n_benchmarks)
-        benchmark_names = list(six.iterkeys(self.benchmarks))
 
-        # We'll generally try to find the tupels p, i, j, k such that matrix[i, p, j] = - 1 / eps and matrix[i, p, i] = 1 / eps
+        benchmark_names = list(self.benchmarks.keys())
 
-        for i, benchmark in enumerate(six.iterkeys(self.benchmarks)):
-            # For the FD-shited benchmarks, we assume that the gradients are the same as at the original point, and will just copy the matrix later
+        # We'll generally try to find the tuples p, i, j, k such that
+        # matrix[i, p, j] = - 1 / eps and matrix[i, p, i] = 1 / eps
+        for i, benchmark in enumerate(self.benchmarks.keys()):
+            # For the FD-shifted benchmarks, we assume that the gradients are
+            # the same as at the original point, and will just copy the matrix later
             copy_to = []
             if benchmark not in self.finite_difference_benchmarks:
                 continue
 
-            for p, param in enumerate(six.iterkeys(self.parameters)):
+            for p, param in enumerate(self.parameters.keys()):
                 shifted_benchmark = self.finite_difference_benchmarks[benchmark][param]
                 j = benchmark_names.index(shifted_benchmark)
                 copy_to.append(j)
@@ -640,7 +652,6 @@ class DataAnalyzer(object):
         n_events, _ = benchmark_weights.shape
 
         # Inputs
-        include_nuisance_benchmarks = nus is not None
         if nus is None:
             nus = [None for _ in thetas]
         assert len(nus) == len(thetas), "Numbers of thetas and nus don't match!"
@@ -765,7 +776,7 @@ class DataAnalyzer(object):
                 end_event = int(round((1.0 - test_split) * self.n_samples, 0))
                 correction_factor = 1.0 / (1.0 - test_split)
                 if end_event < 0 or end_event > self.n_samples:
-                    raise ValueError("Irregular train / test split: sample {} / {}", end_event, self.n_samples)
+                    raise ValueError(f"Irregular split: sample {end_event} / {self.n_samples}")
 
         else:
             if test_split is None or test_split <= 0.0 or test_split >= 1.0:
@@ -775,7 +786,7 @@ class DataAnalyzer(object):
                 start_event = int(round((1.0 - test_split) * self.n_samples, 0)) + 1
                 correction_factor = 1.0 / test_split
                 if start_event < 0 or start_event > self.n_samples:
-                    raise ValueError("Irregular train / test split: sample {} / {}", start_event, self.n_samples)
+                    raise ValueError(f"Irregular split: sample {start_event} / {self.n_samples}")
 
             end_event = None
 
@@ -826,9 +837,7 @@ class DataAnalyzer(object):
                 correction_factor = 1.0 / train_split
 
                 if end_event < 0 or end_event > self.n_samples:
-                    raise ValueError(
-                        "Irregular train / validation / test split: sample {} / {}", end_event, self.n_samples
-                    )
+                    raise ValueError(f"Irregular split: sample {end_event} / {self.n_samples}")
 
         elif partition == "validation":
             if validation_split is None or validation_split <= 0.0 or validation_split >= 1.0:
@@ -842,13 +851,10 @@ class DataAnalyzer(object):
                 correction_factor = 1.0 / validation_split
 
                 if start_event < 0 or start_event > self.n_samples:
-                    raise ValueError(
-                        "Irregular train / validation / test  split: sample {} / {}", start_event, self.n_samples
-                    )
+                    raise ValueError(f"Irregular split: sample {start_event} / {self.n_samples}")
+
                 if end_event < 0 or end_event > self.n_samples:
-                    raise ValueError(
-                        "Irregular train / validation / test split: sample {} / {}", end_event, self.n_samples
-                    )
+                    raise ValueError(f"Irregular split: sample {end_event} / {self.n_samples}")
 
         elif partition == "test":
             end_event = None
@@ -860,17 +866,15 @@ class DataAnalyzer(object):
                 start_event = int(round((1.0 - test_split) * self.n_samples, 0)) + 1
                 correction_factor = 1.0 / test_split
                 if start_event < 0 or start_event > self.n_samples:
-                    raise ValueError(
-                        "Irregular train / validation / test split: sample {} / {}", start_event, self.n_samples
-                    )
+                    raise ValueError(f"Irregular split: sample {start_event} / {self.n_samples}")
 
         else:
-            raise RuntimeError("Unknown partition {}, has to be 'train', 'validation', or 'test'.")
+            raise RuntimeError(f"Unknown partition {partition}")
 
         return start_event, end_event, correction_factor
 
     def _get_theta_value(self, theta):
-        if isinstance(theta, six.string_types):
+        if isinstance(theta, str):
             benchmark = self.benchmarks[theta]
             theta_value = np.array([benchmark[key] for key in benchmark])
         elif isinstance(theta, int):
@@ -895,7 +899,7 @@ class DataAnalyzer(object):
             theta_matrix = np.zeros(self.n_benchmarks)
             theta_matrix[: unpadded_theta_matrix.shape[0]] = unpadded_theta_matrix
 
-        elif isinstance(theta, six.string_types):
+        elif isinstance(theta, str):
             i_benchmark = list(self.benchmarks).index(theta)
             theta_matrix = self._get_theta_benchmark_matrix(i_benchmark)
 
@@ -919,18 +923,18 @@ class DataAnalyzer(object):
             dtheta_matrix = np.zeros((unpadded_theta_matrix.shape[0], self.n_benchmarks))
             dtheta_matrix[:, : unpadded_theta_matrix.shape[1]] = unpadded_theta_matrix
 
-        elif isinstance(theta, six.string_types) and mode == "morphing":
+        elif isinstance(theta, str) and mode == "morphing":
             benchmark = self.benchmarks[theta]
-            benchmark = np.array([value for _, value in six.iteritems(benchmark)])
+            benchmark = np.array([value for _, value in benchmark.items()])
             dtheta_matrix = self._get_dtheta_benchmark_matrix(benchmark)
 
         elif isinstance(theta, int) and mode == "morphing":
             benchmark = self.benchmarks[list(self.benchmarks.keys())[theta]]
-            benchmark = np.array([value for _, value in six.iteritems(benchmark)])
+            benchmark = np.array([value for _, value in benchmark.items()])
             dtheta_matrix = self._get_dtheta_benchmark_matrix(benchmark)
 
-        elif isinstance(theta, six.string_types):  # finite differences
-            benchmark_id = list(six.iterkeys(self.benchmarks)).index(theta)
+        elif isinstance(theta, str):
+            benchmark_id = list(self.benchmarks.keys()).index(theta)
             dtheta_matrix = self._get_dtheta_benchmark_matrix(benchmark_id)
 
         elif isinstance(theta, int):  # finite differences
@@ -942,15 +946,14 @@ class DataAnalyzer(object):
             if mode == "fd":
                 raise RuntimeError("Cannot calculate score for arbitrary parameter points without morphing setup")
 
-            dtheta_matrix = self.morpher.calculate_morphing_weight_gradient(
-                theta
-            )  # Shape (n_parameters, n_benchmarks_phys)
+            # Shape (n_parameters, n_benchmarks_phys)
+            dtheta_matrix = self.morpher.calculate_morphing_weight_gradient(theta)
 
         return dtheta_matrix
 
     def _calculate_sampling_factors(self):
         events = np.asarray(self.n_events_generated_per_benchmark, dtype=np.float)
-        logger.debug("Events per benchmark: %s", events)
+        logger.debug(f"Events per benchmark: {events}")
         factors = events / np.sum(events)
         factors = np.hstack((factors, 1.0))  # background events
         return factors
@@ -970,7 +973,6 @@ class DataAnalyzer(object):
         return closest_idx
 
     def _benchmark_array(self):
-        benchmarks_array = []
-        for benchmark in six.itervalues(self.benchmarks):
-            benchmarks_array.append(list(six.itervalues(benchmark)))
-        return np.asarray(benchmarks_array)
+        return np.asarray([
+            list(benchmark.values()) for benchmark in self.benchmarks.values()
+        ])
