@@ -1,8 +1,7 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import logging
 import numpy as np
 import random
+
 from scipy.interpolate import griddata, LinearNDInterpolator, CloughTocher2DInterpolator
 from scipy.stats import chi2
 
@@ -12,7 +11,6 @@ logger = logging.getLogger(__name__)
 
 
 class InformationGeometry:
-
     """
     Functions to calculate limits using Information Geometry.
 
@@ -20,13 +18,12 @@ class InformationGeometry:
     one of the following functions
 
     * `InformationGeometry.information_from_formula()` defines the Fisher Information
-    explicitly as function of the theory paramters `theta`.
-    * `InformationGeometry.information_from_grid()` loads a grid of Fisher Informations
-    which is then interpolated.
+      explicitly as function of the theory parameters `theta`.
+    * `InformationGeometry.information_from_grid()` loads a grid of Fisher Information
+      which is then interpolated.
 
     Using information geometrical methods, the function `InformationGeometry.distance_contours()` then
     calculates the distance contours and equivalently the p-values throughout parameter space.
-
     """
 
     def __init__(self):
@@ -38,7 +35,7 @@ class InformationGeometry:
     def information_from_formula(self, formula, dimension):
         """
         Explicitly defines the Fisher Information as function of the theory parameter `theta`
-        through a formula that can be avaulated using `eval()`.
+        through a formula that can be evaluated using `eval()`.
 
         Parameters
         ----------
@@ -93,7 +90,7 @@ class InformationGeometry:
         elif option == "smooth":
             self.infofunction = CloughTocher2DInterpolator(points=theta_grid, values=np.array(fisherinformation_grid))
         else:
-            RuntimeError("Option %s unknown", option)
+            RuntimeError(f"Option unknown: {option}")
 
         # Interpolate inverse information
         if self.inverse == "interpolate":
@@ -220,7 +217,7 @@ class InformationGeometry:
 
         limits : list of (tuple of float)
             Specifies the boundaries of the parameter grid in which the trajectory
-            is evaulated. It should be `[[min, max], [min, max], ..., [min, max]`,
+            is evaluated. It should be `[[min, max], [min, max], ..., [min, max]`,
             where the list goes over all parameters and `min` and `max` are float.
 
         stepsize : int, optional
@@ -300,7 +297,7 @@ class InformationGeometry:
 
         grid_ranges : list of (tuple of float)
             Specifies the boundaries of the parameter grid in which the trajectory
-            is evaulated. It should be `[[min, max], [min, max], ..., [min, max]`,
+            is evaluated. It should be `[[min, max], [min, max], ..., [min, max]`,
             where the list goes over all parameters and `min` and `max` are float.
 
         grid_resolutions : list of int
@@ -342,12 +339,13 @@ class InformationGeometry:
         """
 
         # automatic setting of stepsize and ntrajectories
-        if stepsize == None:
+        if stepsize is None:
             stepsize = min([(limit[1] - limit[0]) / 20.0 for limit in grid_ranges])
-        if ntrajectories == None:
+        if ntrajectories is None:
             ntrajectories = 20 * self.dimension
         if self.dimension is not 2:
             continous_sampling = False
+
         limits = (1.0 + 2.0 * stepsize) * np.array(grid_ranges)
 
         # determine trajectories
@@ -359,7 +357,8 @@ class InformationGeometry:
                 dth0 = np.array([np.cos(angle), np.sin(angle)])
             else:
                 dth0 = np.array([random.uniform(-1, 1) for _ in range(self.dimension)])
-            logger.debug("Calculate Trajectory Number %s with dtheta0=%s", i, dth0)
+
+            logger.debug(f"Calculate Trajectory Number {i} with dtheta0={dth0}")
             ths, ds = self.find_trajectory(theta0, dth0, limits, stepsize)
             for th in ths:
                 thetas.append(th)
