@@ -1,11 +1,9 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-import six
-
-import numpy as np
-from collections import OrderedDict
-import uproot
-import os
 import logging
+import numpy as np
+import os
+import uproot3
+from collections import OrderedDict
+from particle import Particle
 
 from madminer.utils.particle import MadMinerParticle
 from madminer.utils.various import math_commands
@@ -42,15 +40,12 @@ def parse_delphes_root_file(
         logger.debug("Extracting weights %s", weight_labels)
 
     # Delphes ROOT file
-    root_file = uproot.open(str(delphes_sample_file))
-    # The str() call is important when using numpy 1.16.0 and Python 2.7. In this combination of versions, a unicode
-    # delphes_sample_file would lead to a crash.
+    root_file = uproot3.open(delphes_sample_file)
 
     # Delphes tree
     tree = root_file["Delphes"]
 
     # Weights
-    n_weights = 0
     weights = None
     if weight_labels is not None:
         try:
@@ -97,7 +92,7 @@ def parse_delphes_root_file(
 
     # Prepare variables
     def get_objects(ievent):
-        visible_momentum = MadMinerParticle()
+        visible_momentum = MadMinerParticle.from_xyzt(0.0, 0.0, 0.0, 0.0)
         for p in (
             electrons_all_events[ievent]
             + jets_all_events[ievent]
@@ -118,7 +113,7 @@ def parse_delphes_root_file(
                 "met": met_all_events[ievent][0],
                 "visible": visible_momentum,
                 "all": all_momentum,
-                "boost_to_com": lambda momentum: momentum.boost(all_momentum.boost_vector()),
+                "boost_to_com": lambda momentum: momentum.boost(all_momentum.to_Vector3D()),
             }
         )
 
@@ -127,14 +122,14 @@ def parse_delphes_root_file(
     # Observations
     observable_values = OrderedDict()
 
-    for obs_name, obs_definition in six.iteritems(observables):
+    for obs_name, obs_definition in observables.items():
         values_this_observable = []
 
         # Loop over events
         for event in range(n_events):
             variables = get_objects(event)
 
-            if isinstance(obs_definition, six.string_types):
+            if isinstance(obs_definition, str):
                 try:
                     values_this_observable.append(eval(obs_definition, variables))
                 except (SyntaxError, NameError, TypeError, ZeroDivisionError, IndexError):
@@ -187,7 +182,7 @@ def parse_delphes_root_file(
     # Check for existence of required observables
     combined_filter = None
 
-    for obs_name, obs_required in six.iteritems(observables_required):
+    for obs_name, obs_required in observables_required.items():
         if obs_required:
             this_filter = np.isfinite(observable_values[obs_name])
             n_pass = np.sum(this_filter)
@@ -271,11 +266,10 @@ def _get_particles_truth(tree, pt_min, eta_max, included_pdgids=None):
                 continue
             if eta_max is not None and abs(eta) > eta_max:
                 continue
-            if (included_pdgids is not None) and (not pdgid in included_pdgids):
+            if (included_pdgids is not None) and (pdgid not in included_pdgids):
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphie(pt, eta, phi, e)
+            particle = MadMinerParticle.from_rhophietat(pt, phi, eta, e)
             particle.set_pdgid(pdgid)
             event_particles.append(particle)
 
@@ -285,10 +279,10 @@ def _get_particles_truth(tree, pt_min, eta_max, included_pdgids=None):
 
 
 def _get_particles_charged(tree, name, mass, pdgid_positive_charge, pt_min, eta_max):
-    pts = tree.array(name + ".PT")
-    etas = tree.array(name + ".Eta")
-    phis = tree.array(name + ".Phi")
-    charges = tree.array(name + ".Charge")
+    pts = tree.array(f"{name}.PT")
+    etas = tree.array(f"{name}.Eta")
+    phis = tree.array(f"{name}.Phi")
+    charges = tree.array(f"{name}.Charge")
 
     all_particles = []
 
@@ -304,8 +298,7 @@ def _get_particles_charged(tree, name, mass, pdgid_positive_charge, pt_min, eta_
 
             pdgid = pdgid_positive_charge if charge >= 0.0 else -pdgid_positive_charge
 
-            particle = MadMinerParticle()
-            particle.setptetaphim(pt, eta, phi, mass)
+            particle = MadMinerParticle.from_rhophietatau(pt, phi, eta, mass)
             particle.set_pdgid(pdgid)
             event_particles.append(particle)
 
@@ -315,10 +308,13 @@ def _get_particles_charged(tree, name, mass, pdgid_positive_charge, pt_min, eta_
 
 
 def _get_particles_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
+    ids_mu = {int(p.pdgid) for p in Particle.findall(pdg_name="mu")}
     pt_mu = tree.array("Muon.PT")
     eta_mu = tree.array("Muon.Eta")
     phi_mu = tree.array("Muon.Phi")
     charge_mu = tree.array("Muon.Charge")
+
+    ids_e = {int(p.pdgid) for p in Particle.findall(pdg_name="e")}
     pt_e = tree.array("Electron.PT")
     eta_e = tree.array("Electron.Eta")
     phi_e = tree.array("Electron.Phi")
@@ -352,13 +348,13 @@ def _get_particles_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
 
             pdgid = pdgid_positive_charge if charge >= 0.0 else -pdgid_positive_charge
 
-            if abs(int(pdgid)) == 11:
+            if int(pdgid) in ids_e:
                 if pt_min_e is not None and pt < pt_min_e:
                     continue
                 if eta_max_e is not None and abs(eta) > eta_max_e:
                     continue
 
-            elif abs(int(pdgid)) == 13:
+            elif int(pdgid) in ids_mu:
                 if pt_min_mu is not None and pt < pt_min_mu:
                     continue
                 if eta_max_mu is not None and abs(eta) > eta_max_mu:
@@ -368,8 +364,7 @@ def _get_particles_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
                 logger.warning("Delphes ROOT file has lepton with PDG ID %s, ignoring it", pdgid)
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphim(pt, eta, phi, mass)
+            particle = MadMinerParticle.from_rhophietatau(pt, phi, eta, mass)
             particle.set_pdgid(pdgid)
             event_particles.append(particle)
 
@@ -379,6 +374,8 @@ def _get_particles_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
 
 
 def _get_particles_truth_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_mu):
+    ids_e = {int(p.pdgid) for p in Particle.findall(pdg_name="e")}
+    ids_mu = {int(p.pdgid) for p in Particle.findall(pdg_name="mu")}
     es = tree.array("Particle.E")
     pts = tree.array("Particle.PT")
     etas = tree.array("Particle.Eta")
@@ -392,19 +389,18 @@ def _get_particles_truth_leptons(tree, pt_min_e, eta_max_e, pt_min_mu, eta_max_m
         event_particles = []
 
         for e, pt, eta, phi, pdgid in zip(es[ievent], pts[ievent], etas[ievent], phis[ievent], pdgids[ievent]):
-            if pdgid not in [11, 13, -11, -13]:
+            if pdgid not in {*ids_e, *ids_mu}:
                 continue
-            if pdgid in [11, -11] and (pt_min_e is not None and pt < pt_min_e):
+            if pdgid in ids_e and (pt_min_e is not None and pt < pt_min_e):
                 continue
-            if pdgid in [11, -11] and (eta_max_e is not None and abs(eta) > eta_max_e):
+            if pdgid in ids_e and (eta_max_e is not None and abs(eta) > eta_max_e):
                 continue
-            if pdgid in [13, -13] and (pt_min_mu is not None and pt < pt_min_mu):
+            if pdgid in ids_mu and (pt_min_mu is not None and pt < pt_min_mu):
                 continue
-            if pdgid in [13, -13] and (eta_max_mu is not None and abs(eta) > eta_max_mu):
+            if pdgid in ids_mu and (eta_max_mu is not None and abs(eta) > eta_max_mu):
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphie(pt, eta, phi, e)
+            particle = MadMinerParticle.from_rhophietat(pt, phi, eta, e)
             particle.set_pdgid(pdgid)
             event_particles.append(particle)
 
@@ -431,8 +427,7 @@ def _get_particles_photons(tree, pt_min, eta_max):
             if eta_max is not None and abs(eta) > eta_max:
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphie(pt, eta, phi, e)
+            particle = MadMinerParticle.from_rhophietat(pt, phi, eta, e)
             particle.set_pdgid(22)
             event_particles.append(particle)
 
@@ -450,12 +445,12 @@ def _get_particles_jets(tree, pt_min, eta_max):
         tau_tags = tree.array("Jet.TauTag")
     except:
         logger.warning("Did not find tau-tag information in Delphes ROOT file.")
-        tau_tags = [0 for _ in range(len(pts))]
+        tau_tags = [0] * len(pts)
     try:
         b_tags = tree.array("Jet.BTag")
     except:
         logger.warning("Did not find b-tag information in Delphes ROOT file.")
-        b_tags = [0 for _ in range(len(pts))]
+        b_tags = [0] * len(pts)
 
     all_particles = []
 
@@ -471,8 +466,7 @@ def _get_particles_jets(tree, pt_min, eta_max):
             if eta_max is not None and abs(eta) > eta_max:
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphim(pt, eta, phi, mass)
+            particle = MadMinerParticle.from_rhophietatau(pt, phi, eta, mass)
             particle.set_pdgid(9)
             particle.set_tags(tau_tag >= 1, b_tag >= 1, False)
             event_particles.append(particle)
@@ -491,12 +485,12 @@ def _get_particles_truth_jets(tree, pt_min, eta_max):
         tau_tags = tree.array("GenJet.TauTag")
     except:
         logger.warning("Did not find tau-tag information for GenJets in Delphes ROOT file.")
-        tau_tags = [0 for _ in range(len(pts))]
+        tau_tags = [0] * len(pts)
     try:
         b_tags = tree.array("GenJet.BTag")
     except:
         logger.warning("Did not find b-tag information for GenJets in Delphes ROOT file.")
-        b_tags = [0 for _ in range(len(pts))]
+        b_tags = [0] * len(pts)
 
     all_particles = []
 
@@ -512,8 +506,7 @@ def _get_particles_truth_jets(tree, pt_min, eta_max):
             if eta_max is not None and abs(eta) > eta_max:
                 continue
 
-            particle = MadMinerParticle()
-            particle.setptetaphim(pt, eta, phi, mass)
+            particle = MadMinerParticle.from_rhophietatau(pt, phi, eta, mass)
             particle.set_pdgid(9)
             particle.set_tags(tau_tag >= 1, b_tag >= 1, False)
             event_particles.append(particle)
@@ -533,8 +526,7 @@ def _get_particles_truth_met(tree):
         event_particles = []
 
         for met, phi in zip(mets[ievent], phis[ievent]):
-            particle = MadMinerParticle()
-            particle.setptetaphim(met, 0.0, phi, 0.0)
+            particle = MadMinerParticle.from_rhophietatau(met, phi, 0.0, 0.0)
             particle.set_pdgid(0)
             event_particles.append(particle)
 
@@ -553,8 +545,7 @@ def _get_particles_met(tree):
         event_particles = []
 
         for met, phi in zip(mets[ievent], phis[ievent]):
-            particle = MadMinerParticle()
-            particle.setptetaphim(met, 0.0, phi, 0.0)
+            particle = MadMinerParticle.from_rhophietatau(met, phi, 0.0, 0.0)
             particle.set_pdgid(0)
             event_particles.append(particle)
 
