@@ -1,22 +1,18 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import six
-import numpy as np
-from collections import OrderedDict
-import os
 import logging
+import numpy as np
+import os
+from collections import OrderedDict
+from particle import Particle
 
 try:
     import xml.etree.cElementTree as ET
-
     use_celementtree = True
 except ImportError:
     import xml.etree.ElementTree as ET
-
     use_celementtree = False
 
-from madminer.utils.various import unzip_file, approx_equal, math_commands
 from madminer.utils.particle import MadMinerParticle
+from madminer.utils.various import unzip_file, approx_equal, math_commands
 
 logger = logging.getLogger(__name__)
 
@@ -56,21 +52,21 @@ def parse_lhe_file(
     if k_factor is None:
         k_factor = 1.0
     if observables_required is None:
-        observables_required = {key: False for key in six.iterkeys(observables)}
+        observables_required = {key: False for key in observables.keys()}
     if observables_defaults is None:
-        observables_defaults = {key: None for key in six.iterkeys(observables)}
+        observables_defaults = {key: None for key in observables.keys()}
     if is_background and benchmark_names is None:
         raise RuntimeError("Parsing background LHE files required benchmark names to be provided.")
     if cuts is None:
         cuts = OrderedDict()
     if cuts_default_pass is None:
-        cuts_default_pass = {key: False for key in six.iterkeys(cuts)}
+        cuts_default_pass = {key: False for key in cuts.keys()}
     if efficiencies is None:
         efficiencies = OrderedDict()
     if efficiencies_default_pass is None:
-        efficiencies_default_pass = {key: 1.0 for key in six.iterkeys(efficiencies)}
+        efficiencies_default_pass = {key: 1.0 for key in efficiencies.keys()}
 
-    # Untar and open LHE file
+    # Unzip and open LHE file
     run_card = None
     for elem in _untar_and_parse_lhe_file(filename):
         if elem.tag == "MGRunCard":
@@ -260,11 +256,12 @@ def parse_lhe_file(
             output_weights[benchmark_name] = weights_all_events[sampling_benchmark]
         else:
             output_weights[benchmark_name] = weights_all_events[benchmark_name]
-    for syst_name, syst_data in six.iteritems(systematics_dict):
+
+    for syst_name, syst_data in systematics_dict.items():
         for (
             nuisance_param_name,
             ((nuisance_benchmark0, weight_name0), (nuisance_benchmark1, weight_name1), processing),
-        ) in six.iteritems(syst_data):
+        ) in syst_data.items():
             # Store first benchmark associated with nuisance param
             if weight_name0 is None:
                 weight_name0 = sampling_benchmark
@@ -273,7 +270,7 @@ def parse_lhe_file(
             elif isinstance(processing, float):
                 output_weights[nuisance_benchmark0] = processing * weights_all_events[weight_name0]
             else:
-                raise RuntimeError("Unknown nuisance processiing {}".format(processing))
+                raise RuntimeError(f"Unknown nuisance processing {processing}")
 
             # Store second benchmark associated with nuisance param
             if nuisance_benchmark1 is None or weight_name1 is None:
@@ -283,7 +280,7 @@ def parse_lhe_file(
             elif isinstance(processing, float):
                 output_weights[nuisance_benchmark1] = processing * weights_all_events[weight_name1]
             else:
-                raise RuntimeError("Unknown nuisance processing {}".format(processing))
+                raise RuntimeError(f"Unknown nuisance processing {processing}")
 
     return observations_dict, output_weights
 
@@ -305,11 +302,14 @@ def _report_parse_results(
         logger.info("  %s / %s events pass efficiency %s", n_pass, n_pass + n_fail, efficiency)
     for n_eff, efficiency, n_pass, n_fail in zip(avg_efficiencies, efficiencies, pass_efficiencies, fail_efficiencies):
         logger.info("  average efficiency for %s is %s", efficiency, n_eff / (n_pass + n_fail))
+
     n_events_pass = len(observations_all_events)
+
     if len(cuts) > 0:
         logger.info("  %s events pass all cuts/efficiencies", n_events_pass)
     if n_events_with_negative_weights > 0:
         logger.warning("  %s events contain negative weights", n_events_with_negative_weights)
+
     return n_events_pass
 
 
@@ -365,14 +365,26 @@ def _parse_event(
     pass_all_cuts = True
     if pass_all_observation:
         pass_all_cuts = _parse_cuts(
-            cuts, cuts_default_pass, fail_cuts, observables, observations, pass_all_cuts, pass_cuts, variables
+            cuts,
+            cuts_default_pass,
+            fail_cuts,
+            observables,
+            observations,
+            pass_all_cuts,
+            pass_cuts,
+            variables,
         )
 
     # Efficiencies
     pass_all_efficiencies = True
     if pass_all_observation and pass_all_cuts:
         pass_all_efficiencies, total_efficiency = _parse_efficiencies(
-            avg_efficiencies, efficiencies, efficiencies_default_pass, fail_efficiencies, pass_efficiencies, variables
+            avg_efficiencies,
+            efficiencies,
+            efficiencies_default_pass,
+            fail_efficiencies,
+            pass_efficiencies,
+            variables,
         )
 
         if pass_all_efficiencies:
@@ -395,20 +407,23 @@ def _parse_event(
 
 def _report_negative_weights(n_events_with_negative_weights, weights):
     n_negative_weights = np.sum(np.array(list(weights.values())) < 0.0)
+
     if n_negative_weights > 0:
         n_events_with_negative_weights += 1
         if n_events_with_negative_weights <= 3:
             logger.warning("Found %s negative weights in event. Weights: %s", n_negative_weights, weights)
         if n_events_with_negative_weights == 3:
             logger.warning("Skipping warnings about negative weights from now on...")
+
     return n_events_with_negative_weights
 
 
 def _parse_observations(observables, observables_defaults, observables_required, variables):
     observations = []
     pass_all_observation = True
-    for obs_name, obs_definition in six.iteritems(observables):
-        if isinstance(obs_definition, six.string_types):
+
+    for obs_name, obs_definition in observables.items():
+        if isinstance(obs_definition, str):
             try:
                 observations.append(eval(obs_definition, variables))
             except (SyntaxError, NameError, TypeError, ZeroDivisionError, IndexError):
@@ -434,15 +449,22 @@ def _parse_observations(observables, observables_defaults, observables_required,
                 if default is None:
                     default = np.nan
                 observations.append(default)
+
     return observations, pass_all_observation
 
 
 def _parse_efficiencies(
-    avg_efficiencies, efficiencies, efficiencies_default_pass, fail_efficiencies, pass_efficiencies, variables
+    avg_efficiencies,
+    efficiencies,
+    efficiencies_default_pass,
+    fail_efficiencies,
+    pass_efficiencies,
+    variables,
 ):
     # Apply efficiencies
     total_efficiency = 1.0
     pass_all_efficiencies = True
+
     for i_efficiency, (efficiency, default_pass) in enumerate(zip(efficiencies, efficiencies_default_pass)):
         try:
             efficiency_result = eval(efficiency, variables)
@@ -462,6 +484,7 @@ def _parse_efficiencies(
             else:
                 fail_efficiencies[i_efficiency] += 1
                 pass_all_efficiencies = False
+
     return pass_all_efficiencies, total_efficiency
 
 
@@ -469,6 +492,7 @@ def _parse_cuts(cuts, cuts_default_pass, fail_cuts, observables, observations, p
     # Objects for cuts
     for obs_name, obs_value in zip(observables.keys(), observations):
         variables[obs_name] = obs_value
+
     # Check cuts
     for i_cut, (cut, default_pass) in enumerate(zip(cuts, cuts_default_pass)):
         try:
@@ -485,6 +509,7 @@ def _parse_cuts(cuts, cuts_default_pass, fail_cuts, observables, observations, p
             else:
                 fail_cuts[i_cut] += 1
                 pass_all_cuts = False
+
     return pass_all_cuts
 
 
@@ -504,7 +529,7 @@ def extract_nuisance_parameters_from_lhe_file(filename, systematics):
     # Parse scale factors from strings in systematics
     logger.debug("Systematics setup: %s", systematics)
 
-    # Untar and parse LHE file
+    # Unzip and parse LHE file
     initrwgts = _untar_and_parse_lhe_file(filename, ["initrwgt"])
 
     # Find weight groups
@@ -519,7 +544,7 @@ def extract_nuisance_parameters_from_lhe_file(filename, systematics):
     logger.debug("%s weight groups", len(weight_groups))
 
     # Loop over systematics
-    for syst_name, syst_value in six.iteritems(systematics):
+    for syst_name, syst_value in systematics.items():
         nuisance_param_dict = _extract_nuisance_param_dict(weight_groups, syst_name, syst_value)
         systematics_dict[syst_name] = nuisance_param_dict
 
@@ -532,8 +557,8 @@ def _extract_nuisance_param_dict(weight_groups, systematics_name, systematics_de
     syst_type = systematics_definition[0]
 
     if syst_type == "norm":
-        nuisance_param_name = "{}_nuisance_param_0".format(systematics_name)
-        benchmark_name = "{}_benchmark_0".format(nuisance_param_name)
+        nuisance_param_name = f"{systematics_name}_nuisance_param_0"
+        benchmark_name = f"{nuisance_param_name}_benchmark_0"
         nuisance_param_definition = (benchmark_name, None), (None, None), systematics_definition[1]
         return {nuisance_param_name: nuisance_param_definition}
 
@@ -596,7 +621,7 @@ def _extract_nuisance_param_dict(weight_groups, systematics_name, systematics_de
 
                     # Matching time!
                     if approx_equal(weight_mur, mur) and approx_equal(weight_muf, muf):
-                        benchmark_name = "{}_nuisance_param_0_benchmark_{}".format(systematics_name, k)
+                        benchmark_name = f"{systematics_name}_nuisance_param_0_benchmark_{k}"
                         nuisance_param_definition_parts.append((benchmark_name, weight_id))
                         break
 
@@ -614,7 +639,7 @@ def _extract_nuisance_param_dict(weight_groups, systematics_name, systematics_de
             return {}
         else:
             # Output
-            nuisance_param_name = "{}_nuisance_param_0".format(systematics_name)
+            nuisance_param_name = f"{systematics_name}_nuisance_param_0"
             if len(nuisance_param_definition_parts) > 1:
                 nuisance_dict = {
                     nuisance_param_name: (nuisance_param_definition_parts[0], nuisance_param_definition_parts[1], None)
@@ -657,8 +682,8 @@ def _extract_nuisance_param_dict(weight_groups, systematics_name, systematics_de
                 logger.debug("Found PDF weight %s / %s", weight_id, weight_pdf)
 
                 # Add every PDF Hessian direction to nuisance parameters
-                nuisance_param_name = "{}_nuisance_param_{}".format(systematics_name, i)
-                benchmark_name = "{}_benchmark_0".format(nuisance_param_name)
+                nuisance_param_name = f"{systematics_name}_nuisance_param_{i}"
+                benchmark_name = f"{nuisance_param_name}_benchmark_0"
                 nuisance_dict[nuisance_param_name] = (benchmark_name, weight_id), (None, None), None
 
         # Check that everything was found
@@ -719,8 +744,7 @@ def _parse_xml_event(event, sampling_benchmark):
             pz = float(elements[8])
             e = float(elements[9])
             spin = float(elements[12])
-            particle = MadMinerParticle()
-            particle.setpxpypze(px, py, pz, e)
+            particle = MadMinerParticle.from_xyzt(px, py, pz, e)
             particle.set_pdgid(pdgid)
             particle.set_spin(spin)
             particles.append(particle)
@@ -829,8 +853,7 @@ def _parse_txt_events(filename, sampling_benchmark):
                     pz = float(elements[8])
                     e = float(elements[9])
                     spin = float(elements[12])
-                    particle = MadMinerParticle()
-                    particle.setpxpypze(px, py, pz, e)
+                    particle = MadMinerParticle.from_xyzt(px, py, pz, e)
                     particle.set_pdgid(pdgid)
                     particle.set_spin(spin)
                     particles.append(particle)
@@ -854,7 +877,7 @@ def _parse_lhe_file_with_bad_chars(filename):
 
 
 def _untar_and_parse_lhe_file(filename, tags=None):
-    # Untar event file
+    # Unzip event file
     new_filename, extension = os.path.splitext(filename)
     if extension == ".gz":
         if not os.path.exists(new_filename):
@@ -920,12 +943,17 @@ def _get_objects(particles, particles_truth, met_resolution=None, global_event_d
 
     # Sum over all particles
     ht = 0.0
-    visible_sum = MadMinerParticle()
-    visible_sum.setpxpypze(0.0, 0.0, 0.0, 0.0)
+    visible_sum = MadMinerParticle.from_xyzt(0.0, 0.0, 0.0, 0.0)
+
+    standard_ids = set(get_elementary_pdg_ids())
+    neutrino_ids = {int(p.pdgid) for p in [
+        *Particle.findall(pdg_name="nu(e)"),
+        *Particle.findall(pdg_name="nu(mu)"),
+        *Particle.findall(pdg_name="nu(tau)"),
+    ]}
 
     for particle in particles:
-        pdgid = abs(particle.pdgid)
-        if pdgid in [1, 2, 3, 4, 5, 6, 9, 11, 13, 15, 21, 22, 23, 24, 25]:
+        if particle.pdgid in standard_ids and particle.pdgid not in neutrino_ids:
             visible_sum += particle
             ht += particle.pt
 
@@ -939,10 +967,14 @@ def _get_objects(particles, particles_truth, met_resolution=None, global_event_d
         noise_y = 0.0
 
     # MET
-    met_x = -visible_sum.px + noise_x
-    met_y = -visible_sum.py + noise_y
-    met = MadMinerParticle()
-    met.setpxpypze(met_x, met_y, 0.0, (met_x ** 2 + met_y ** 2) ** 0.5)
+    met_x = -visible_sum.x + noise_x
+    met_y = -visible_sum.y + noise_y
+    met = MadMinerParticle.from_xyzt(
+        x=met_x,
+        y=met_y,
+        z=0.0,
+        t=(met_x ** 2 + met_y ** 2) ** 0.5,
+    )
 
     # Build objects
     objects = math_commands()
@@ -995,10 +1027,10 @@ def _smear_particles(particles, energy_resolutions, pt_resolutions, eta_resoluti
         pdgid = particle.pdgid
 
         if (
-            pdgid not in six.iterkeys(energy_resolutions)
-            or pdgid not in six.iterkeys(pt_resolutions)
-            or pdgid not in six.iterkeys(eta_resolutions)
-            or pdgid not in six.iterkeys(phi_resolutions)
+            pdgid not in energy_resolutions.keys()
+            or pdgid not in pt_resolutions.keys()
+            or pdgid not in eta_resolutions.keys()
+            or pdgid not in phi_resolutions.keys()
         ):
             continue
 
@@ -1024,18 +1056,15 @@ def _smear_particles(particles, energy_resolutions, pt_resolutions, eta_resoluti
             while pt <= min_pt:
                 pt = _smear_variable(particle.pt, pt_resolutions, pdgid)
         eta = _smear_variable(particle.eta, eta_resolutions, pdgid)
-        phi = _smear_variable(particle.phi(), phi_resolutions, pdgid)
+        phi = _smear_variable(particle.phi, phi_resolutions, pdgid)
         while phi > 2.0 * np.pi:
             phi -= 2.0 * np.pi
         while phi < 0.0:
             phi += 2.0 * np.pi
 
-        # Construct particle
-        smeared_particle = MadMinerParticle()
-
         if None in energy_resolutions[pdgid]:
             # Calculate E from on-shell conditions
-            smeared_particle.setptetaphim(pt, eta, phi, particle.m)
+            smeared_particle = MadMinerParticle.from_rhophietatau(pt, phi, eta, particle.m)
 
         elif None in pt_resolutions[pdgid]:
             # Calculate pT from on-shell conditions
@@ -1043,21 +1072,29 @@ def _smear_particles(particles, energy_resolutions, pt_resolutions, eta_resoluti
                 pt = (e ** 2 - m ** 2) ** 0.5 / np.cosh(eta)
             else:
                 pt = 0.0
-            smeared_particle.setptetaphie(pt, eta, phi, e)
+            smeared_particle = MadMinerParticle.from_rhophietat(pt, phi, eta, e)
 
         else:
             # Everything smeared manually
-            smeared_particle.setptetaphie(pt, eta, phi, e)
+            smeared_particle = MadMinerParticle.from_rhophietat(pt, phi, eta, e)
 
         # PDG id (also sets charge)
         smeared_particle.set_pdgid(pdgid)
-
         smeared_particles.append(smeared_particle)
 
     return smeared_particles
 
 
 def get_elementary_pdg_ids():
-    ids = [1, -1, 2, -2, 3, -3, 4, -4, 5, -5, 6, -6, 9, 11, -11, 12, -12, 13, -13, 14, -14, 15, -15, 16, -16]
-    ids += [21, 22, 23, 24, -24, 25]
-    return ids
+    """Get Standard Model elementary particle IDs, leaving out 4th generation leptons"""
+    pdg_ids = Particle.findall(
+        lambda p: (
+            p.pdgid.is_quark
+            or p.pdgid.is_lepton
+            or p.pdgid.is_sm_gauge_boson_or_higgs
+        )
+    )
+
+    pdg_ids = [int(p.pdgid) for p in pdg_ids]
+    pdg_ids = [pdg_id for pdg_id in pdg_ids if pdg_id not in {17, -17, 18, -18}]
+    return pdg_ids
