@@ -1,11 +1,9 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import six
-import shutil
 import h5py
-import numpy as np
-from collections import OrderedDict
 import logging
+import numpy as np
+import shutil
+from collections import OrderedDict
+
 
 logger = logging.getLogger(__name__)
 
@@ -58,7 +56,12 @@ def save_nuisance_setup_to_madminer_file(
 
 
 def save_preformatted_events_to_madminer_file(
-    filename, observations, weights, sampling_benchmarks, copy_setup_from, overwrite_existing_samples=True
+    filename,
+    observations,
+    weights,
+    sampling_benchmarks,
+    copy_setup_from,
+    overwrite_existing_samples=True,
 ):
     _copy_madminer_file(copy_setup_from, filename, overwrite_existing_samples)
     _save_events(
@@ -269,6 +272,7 @@ def madminer_event_loader(
 
 def _save_parameters(filename, overwrite_existing_files, parameters):
     io_tag = "w" if overwrite_existing_files else "x"
+
     with h5py.File(filename, io_tag) as f:
         # Prepare parameters
         parameter_names = [pname for pname in parameters]
@@ -293,11 +297,13 @@ def _save_parameters(filename, overwrite_existing_files, parameters):
         f.create_dataset("parameters/max_power", data=parameter_max_power)
         f.create_dataset("parameters/ranges", data=parameter_ranges)
         f.create_dataset("parameters/transforms", (n_parameters,), dtype="S256", data=parameter_transforms)
+
     return parameter_names
 
 
 def _save_benchmarks(benchmarks, benchmarks_is_nuisance, filename, parameter_names):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     with h5py.File(filename, io_tag) as f:
         # Prepare benchmarks
         benchmark_names = [bname for bname in benchmarks]
@@ -320,6 +326,7 @@ def _save_benchmarks(benchmarks, benchmarks_is_nuisance, filename, parameter_nam
 
 def _save_benchmarks2(benchmark_is_nuisance, benchmark_names, benchmark_values, filename, reference_benchmark):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     with h5py.File(filename, io_tag) as f:
         # Prepare benchmarks for saving
         n_benchmarks = len(benchmark_names)
@@ -352,12 +359,12 @@ def _save_finite_differences(filename, finite_difference_benchmarks, finite_diff
     io_tag = "a"  # Read-write if file exists, otherwise create
     with h5py.File(filename, io_tag) as f:
         n_keys = len(finite_difference_benchmarks)
-        n_values = len(finite_difference_benchmarks[list(six.iterkeys(finite_difference_benchmarks))[0]])
+        n_values = len(finite_difference_benchmarks[list(finite_difference_benchmarks.keys())[0]])
 
-        keys_ascii = [key.encode("ascii", "ignore") for key in six.iterkeys(finite_difference_benchmarks)]
+        keys_ascii = [key.encode("ascii", "ignore") for key in finite_difference_benchmarks.keys()]
         values_ascii = [
-            [val.encode("ascii", "ignore") for val in six.itervalues(values)]
-            for values in six.itervalues(finite_difference_benchmarks)
+            [val.encode("ascii", "ignore") for val in values.values()]
+            for values in finite_difference_benchmarks.values()
         ]
 
         f.create_dataset("finite_differences/base_benchmarks", (n_keys,), dtype="S256", data=keys_ascii)
@@ -367,6 +374,7 @@ def _save_finite_differences(filename, finite_difference_benchmarks, finite_diff
 
 def _save_morphing(filename, morphing_components, morphing_matrix):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     with h5py.File(filename, io_tag) as f:
         # Store morphing info
         if morphing_components is not None:
@@ -377,6 +385,7 @@ def _save_morphing(filename, morphing_components, morphing_matrix):
 
 def _save_systematics(filename, systematics):
     io_tag = "a"
+
     with h5py.File(filename, io_tag) as f:
         # Prepare and store systematics setup
         if systematics is not None and len(systematics) > 0:
@@ -393,6 +402,7 @@ def _save_systematics(filename, systematics):
 
 def _save_nuisance_parameters(filename, nuisance_parameters, overwrite_existing_nuisance_parameters):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     with h5py.File(filename, io_tag) as f:
         # Make space for nuisance params
         if overwrite_existing_nuisance_parameters:
@@ -441,6 +451,7 @@ def _save_nuisance_parameters(filename, nuisance_parameters, overwrite_existing_
 
 def _save_n_events(filename, n_events_background, n_events_per_sampling_benchmark, overwrite_existing_samples):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     with h5py.File(filename, io_tag) as f:
         # Check if groups exist already
         if overwrite_existing_samples:
@@ -493,6 +504,7 @@ def _save_events(
 
 def _save_observables(filename, observables, overwrite_existing_samples):
     io_tag = "a"  # Read-write if file exists, otherwise create
+
     if observables is None:
         return None
 
@@ -511,7 +523,7 @@ def _save_observables(filename, observables, overwrite_existing_samples):
         observable_definitions = []
         for key in observable_names:
             definition = observables[key]
-            if isinstance(definition, six.string_types):
+            if isinstance(definition, str):
                 observable_definitions.append(definition.encode("ascii", "ignore"))
             else:
                 observable_definitions.append("".encode("ascii", "ignore"))
@@ -524,6 +536,7 @@ def _save_observables(filename, observables, overwrite_existing_samples):
 
 
 def _load_parameters(filename):
+
     with h5py.File(filename, "r") as f:
         # Parameters
         try:
@@ -556,10 +569,12 @@ def _load_parameters(filename):
 
         except KeyError:
             raise IOError("Cannot read parameters from HDF5 file")
+
     return parameter_names, parameters
 
 
 def _load_benchmarks(filename, include_nuisance_benchmarks, parameter_names, return_dict=True):
+
     with h5py.File(filename, "r") as f:
         # Benchmarks
         try:
@@ -620,6 +635,7 @@ def _load_finite_differences(filename, parameter_names):
 
 
 def _load_n_samples(filename):
+
     with h5py.File(filename, "r") as f:
         # Number of samples
         try:
@@ -645,6 +661,7 @@ def _load_n_samples(filename):
 
 
 def _load_morphing(filename):
+
     with h5py.File(filename, "r") as f:
         # Morphing
         try:
@@ -658,6 +675,7 @@ def _load_morphing(filename):
 
 
 def _load_nuisance_params(filename):
+
     with h5py.File(filename, "r") as f:
         # Nuisance parameters
         try:
@@ -687,10 +705,12 @@ def _load_nuisance_params(filename):
 
         except KeyError:
             nuisance_parameters = None
+
     return nuisance_parameters
 
 
 def _load_systematics(filename):
+
     with h5py.File(filename, "r") as f:
         # Systematics setup
         try:
@@ -711,15 +731,17 @@ def _load_systematics(filename):
                     syst_data[1] = str(syst_data[1])
                     syst_data[2] = str(syst_data[2])
                 else:
-                    raise RuntimeError("Error while reading systematics from HDF5 file: {}".format(syst_data))
+                    raise RuntimeError(f"Error while reading systematics from HDF5 file: {syst_data}")
                 systematics[name] = tuple(syst_data)
 
         except KeyError:
             systematics = OrderedDict()
+
     return systematics
 
 
 def _load_observables(filename):
+
     with h5py.File(filename, "r") as f:
         # Observables
         try:
@@ -734,6 +756,7 @@ def _load_observables(filename):
                 observables[oname] = odef
         except KeyError:
             observables = None
+
     return observables
 
 
@@ -771,6 +794,7 @@ def _sort_weights(benchmark_names, weights):
     except Exception as e:
         logger.warning("Issue matching weight names in HepMC file to benchmark names in MadMiner file:\n%s", e)
         weights_sorted = [weights[key] for key in weights]
+
     return weights_sorted
 
 
