@@ -1,14 +1,13 @@
-from __future__ import absolute_import, division, print_function
-
+import logging
 import numpy as np
 import numpy.random as rng
 import torch
-from torch import tensor
 import torch.nn.functional as F
+
 from madminer.utils.ml.models.base import BaseConditionalFlow
 from madminer.utils.ml.models.masks import create_degrees, create_masks, create_weights_conditional
 from madminer.utils.ml.utils import get_activation_function
-import logging
+from torch import tensor
 
 logger = logging.getLogger(__name__)
 
@@ -26,7 +25,7 @@ class ConditionalMixtureMADE(BaseConditionalFlow):
         input_order="sequential",
         mode="sequential",
     ):
-        super(ConditionalMixtureMADE, self).__init__(n_conditionals, n_inputs)
+        super().__init__(n_conditionals, n_inputs)
 
         # save input arguments
         self.activation = activation
@@ -73,7 +72,6 @@ class ConditionalMixtureMADE(BaseConditionalFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -174,7 +172,8 @@ class ConditionalMixtureMADE(BaseConditionalFlow):
             u = u.to(*self.to_args, **self.to_kwargs)
 
         for i in range(1, self.n_inputs + 1):
-            self.forward(theta, x)  # Sets Gaussian parameters (self.m and self.logp) and mixture coeffs (self.loga)
+            # Sets Gaussian parameters (self.m and self.logp) and mixture coeffs (self.loga)
+            self.forward(theta, x)
 
             ix = np.argwhere(self.input_order == i)[0, 0]
 
@@ -201,7 +200,7 @@ class ConditionalMixtureMADE(BaseConditionalFlow):
         self.to_args = args
         self.to_kwargs = kwargs
 
-        self = super(ConditionalMixtureMADE, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, M in enumerate(self.Ms):
             self.Ms[i] = M.to(*args, **kwargs)
