@@ -1,12 +1,11 @@
-from __future__ import absolute_import, division, print_function
-
+import logging
 import numpy.random as rng
 import torch.nn as nn
+
 from torch import tensor
 from madminer.utils.ml.models.base import BaseFlow, BaseConditionalFlow
 from madminer.utils.ml.models.made import GaussianMADE, ConditionalGaussianMADE
 from madminer.utils.ml.models.batch_norm import BatchNorm
-import logging
 
 logger = logging.getLogger(__name__)
 
@@ -26,7 +25,7 @@ class MaskedAutoregressiveFlow(BaseFlow):
         alpha=0.1,
     ):
 
-        super(MaskedAutoregressiveFlow, self).__init__(n_inputs)
+        super().__init__(n_inputs)
 
         # save input arguments
         self.n_inputs = n_inputs
@@ -103,7 +102,7 @@ class MaskedAutoregressiveFlow(BaseFlow):
         self.to_args = args
         self.to_kwargs = kwargs
 
-        self = super(MaskedAutoregressiveFlow, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, (made) in enumerate(self.mades):
             self.mades[i] = made.to(*args, **kwargs)
@@ -131,7 +130,7 @@ class ConditionalMaskedAutoregressiveFlow(BaseConditionalFlow):
         alpha=0.1,
     ):
 
-        super(ConditionalMaskedAutoregressiveFlow, self).__init__(n_conditionals, n_inputs)
+        super().__init__(n_conditionals, n_inputs)
 
         # save input arguments
         self.n_conditionals = n_conditionals
@@ -157,7 +156,7 @@ class ConditionalMaskedAutoregressiveFlow(BaseConditionalFlow):
             if not (isinstance(input_order, str) and input_order != "random"):
                 input_order = made.input_order[::-1]
 
-        # Batch normalizatino
+        # Batch normalization
         self.bns = None
         if self.batch_norm:
             self.bns = nn.ModuleList()
@@ -220,7 +219,6 @@ class ConditionalMaskedAutoregressiveFlow(BaseConditionalFlow):
              (Default value = None)
         **kwargs :
 
-
         Returns
         -------
 
@@ -252,7 +250,6 @@ class ConditionalMaskedAutoregressiveFlow(BaseConditionalFlow):
 
         **kwargs :
 
-
         Returns
         -------
 
@@ -260,7 +257,7 @@ class ConditionalMaskedAutoregressiveFlow(BaseConditionalFlow):
         self.to_args = args
         self.to_kwargs = kwargs
 
-        self = super(ConditionalMaskedAutoregressiveFlow, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, (made) in enumerate(self.mades):
             self.mades[i] = made.to(*args, **kwargs)
