@@ -1,10 +1,9 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
+import logging
 import torch
 import torch.nn as nn
-from torch.autograd import grad
+
 from madminer.utils.ml.utils import get_activation_function
-import logging
+from torch.autograd import grad
 
 logger = logging.getLogger(__name__)
 
@@ -14,8 +13,7 @@ class DenseLocalScoreModel(nn.Module):
     of Fisher information matrices."""
 
     def __init__(self, n_observables, n_parameters, n_hidden, activation="tanh", dropout_prob=0.0):
-
-        super(DenseLocalScoreModel, self).__init__()
+        super().__init__()
 
         # Save input
         self.n_hidden = n_hidden
@@ -53,16 +51,20 @@ class DenseLocalScoreModel(nn.Module):
 
         # Calculate gradient
         if return_grad_x:
-            x_gradient = grad(t_hat, x, grad_outputs=torch.ones_like(t_hat.data), only_inputs=True, create_graph=True)[
-                0
-            ]
+            x_gradient = grad(
+                t_hat,
+                x,
+                grad_outputs=torch.ones_like(t_hat.data),
+                only_inputs=True,
+                create_graph=True,
+            )[0]
 
             return t_hat, x_gradient
 
         return t_hat
 
     def to(self, *args, **kwargs):
-        self = super(DenseLocalScoreModel, self).to(*args, **kwargs)
+        self = super().to(*args, **kwargs)
 
         for i, layer in enumerate(self.layers):
             self.layers[i] = layer.to(*args, **kwargs)
