@@ -1,31 +1,28 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import six
 import logging
-import os
-import stat
-from subprocess import Popen, PIPE
-import io
-import numpy as np
-import shutil
-from contextlib import contextmanager
 import gzip
+import math
+import numpy as np
+import os
+import shutil
+import stat
+
+from contextlib import contextmanager
+from subprocess import Popen, PIPE
 
 logger = logging.getLogger(__name__)
 
-initialized = False
-
 
 def call_command(cmd, log_file=None, return_std=False):
     if log_file is not None:
-        with io.open(log_file, "wb") as log:
+        with open(log_file, "wb") as log:
             proc = Popen(cmd, stdout=log, stderr=log, shell=True)
             _ = proc.communicate()
             exitcode = proc.returncode
 
         if exitcode != 0:
             raise RuntimeError(
-                "Calling command {} returned exit code {}. Output in file {}.".format(cmd, exitcode, log_file)
+                f"Calling command {cmd} returned exit code {exitcode}. "
+                f"Output in file {log_file}."
             )
     else:
         proc = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
@@ -34,9 +31,12 @@ def call_command(cmd, log_file=None, return_std=False):
 
         if exitcode != 0:
             raise RuntimeError(
-                "Calling command {} returned exit code {}.\n\nStd output:\n\n{}Error output:\n\n{}".format(
-                    cmd, exitcode, out, err
-                )
+                f"Calling command {cmd} returned exit code {exitcode}."
+                f"\n\n"
+                f"Std output: {out}"
+                f"\n\n"
+                f"Error output: {err}"
+                f"\n\n"
             )
 
         if return_std:
@@ -63,13 +63,13 @@ def create_missing_folders(folders):
             os.makedirs(folder)
 
         elif not os.path.isdir(folder):
-            raise OSError("Path {} exists, but is no directory!".format(folder))
+            raise OSError(f"Path {folder} exists, but is no directory!")
 
 
 def format_benchmark(parameters, precision=2):
     output = ""
 
-    for i, (key, value) in enumerate(six.iteritems(parameters)):
+    for i, (key, value) in enumerate(parameters.items()):
         if i > 0:
             output += ", "
 
@@ -101,20 +101,19 @@ def shuffle(*arrays):
 
         if a.shape[0] != n_samples:
             raise RuntimeError(
-                "Mismatching shapes when trying to simultaneously shuffle: {}".format(
-                    [None if val is None else val.shape for val in arrays]
-                )
+                f"Mismatching shapes when trying to simultaneously shuffle: "
+                f"{[None if val is None else val.shape for val in arrays]}"
             )
 
         shuffled_a = a[permutation]
         shuffled_arrays.append(shuffled_a)
-        a = None
 
     return shuffled_arrays
 
 
 def restrict_samplesize(n, *arrays):
     restricted_arrays = []
+
     for i, a in enumerate(arrays):
         if a is None:
             restricted_arrays.append(None)
@@ -156,7 +155,7 @@ def load_and_check(filename, warning_threshold=1.0e9, memmap_files_larger_than_g
     if filename is None:
         return None
 
-    if not isinstance(filename, six.string_types):
+    if not isinstance(filename, str):
         data = filename
         memmap = False
     else:
@@ -182,7 +181,7 @@ def load_and_check(filename, warning_threshold=1.0e9, memmap_files_larger_than_g
         smallest = np.nanmin(data)
         largest = np.nanmax(data)
         if np.abs(smallest) > warning_threshold or np.abs(largest) > warning_threshold:
-            logger.warning("Warning: file %s has some large numbers, rangin from %s to %s", filename, smallest, largest)
+            logger.warning("Warning: file %s has some large numbers, ranging from %s to %s", filename, smallest, largest)
 
     if len(data.shape) == 1:
         data = data.reshape(-1, 1)
@@ -192,34 +191,25 @@ def load_and_check(filename, warning_threshold=1.0e9, memmap_files_larger_than_g
 
 def math_commands():
     """Provides list with math commands - we need this when using eval"""
-
-    from math import acos, asin, atan, atan2, ceil, cos, cosh, exp, floor, log, pi, pow, sin, sinh, sqrt, tan, tanh
-
-    functions = [
-        "acos",
-        "asin",
-        "atan",
-        "atan2",
-        "ceil",
-        "cos",
-        "cosh",
-        "exp",
-        "floor",
-        "log",
-        "pi",
-        "pow",
-        "sin",
-        "sinh",
-        "sqrt",
-        "tan",
-        "tanh",
-    ]
-
-    mathdefinitions = {}
-    for f in functions:
-        mathdefinitions[f] = locals().get(f, None)
-
-    return mathdefinitions
+    return {
+        "acos": math.acos,
+        "asin": math.asin,
+        "atan": math.atan,
+        "atan2": math.atan2,
+        "ceil": math.ceil,
+        "cos": math.cos,
+        "cosh": math.cosh,
+        "exp": math.exp,
+        "floor": math.floor,
+        "log": math.log,
+        "pi": math.pi,
+        "pow": math.pow,
+        "sin": math.sin,
+        "sinh": math.sinh,
+        "sqrt": math.sqrt,
+        "tan": math.tan,
+        "tanh": math.tanh,
+    }
 
 
 def make_file_executable(filename):
@@ -261,8 +251,10 @@ def weighted_quantile(values, quantiles, sample_weight=None, values_sorted=False
     # Input
     values = np.array(values, dtype=np.float64)
     quantiles = np.array(quantiles)
+
     if sample_weight is None:
         sample_weight = np.ones(len(values))
+
     sample_weight = np.array(sample_weight, dtype=np.float64)
     assert np.all(quantiles >= 0.0) and np.all(quantiles <= 1.0), "quantiles should be in [0, 1]"
 
